require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame

  class Game

    attr_reader :title
    attr_reader :players

    def initialize(title)
      @title = title.capitalize
      @players = []
    end

    def load_players(from_file)
      CSV.foreach(from_file) do |row|
        player = Player.new(row[0], row[1].to_i)
        add_player(player)
      end
    end



    def add_player(player)
      @players << player
    end

    def save_high_scores(to_file="high_scores.txt")
      File.open to_file, "w+" do |file|
        file.puts "#{@title} High Scores:"

        @players.sort.each do |player|
         file.puts high_scores_entry(player)
        end
      end
    end



    def high_scores_entry(player)

      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end



    treasures = TreasureTrove::TREASURES
    puts "There are #{treasures.size} treasures to be found:"
    treasures.each do |treasure|
      puts "A #{treasure.name} is worth #{treasure.points} points"
    end

    def print_stats
      strong_players, wimpy_players = @players.partition {|x| x.strong?}
      puts "\n#{strong_players.size} strong players:"
      strong_players.each do |player|
        print_name_and_health(player)
      end

      puts "\n#{wimpy_players.size} wimpy players:"
      wimpy_players.each do |player|
        print_name_and_health(player)
      end


      @players.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.points} grand total points"
      end

      puts "#{total_points} total points from treasures found"

      puts "\n#{title} High Scores:"
      @players.sort.each do |player|
        puts high_scores_entry(player)
      end

    end

    def print_name_and_health(player)
      puts "#{player.name} (#{player.health})"
    end

    def total_points
      @players.reduce (0) {|sum, player| sum + player.points }
    end


    def play(rounds)
      puts @players
      1.upto(rounds) do |round|
        puts "\nRound #{round}:"
        @players.each do |p|
          GameTurn.take_turn(p)
          puts p
        end
      end
    end
  end



  #
  # if __FILE__ ==$0
  #
  # player1 = Player.new("moe")
  # player2 = Player.new("larry", 60)
  # player3 = Player.new("curly", 125)
  #
  # knuckleheads = Game.new("Knuckleheads")
  # knuckleheads.add_player(player1)
  # knuckleheads.add_player(player2)
  # knuckleheads.add_player(player3)
  # puts @players
  #
  # end
end
